package com.hdp.pi.wechat;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;

import com.alibaba.fastjson.JSONObject;
import com.hdp.pi.wechat.dto.WechatMessageDTO;
import com.hdp.pi.wechat.handler.*;

import java.util.concurrent.ConcurrentHashMap;

/**
 * Created by Veal on 2015/5/11.
 */
public class Subscriber {
    public ConcurrentHashMap<String,MessageHandler> handlers = new ConcurrentHashMap<String, MessageHandler>();
    
    public Subscriber(){
    	handlers.put("text", new TextMessageHandler());
    	handlers.put("image", new ImageMessageHandler());
    	handlers.put("voice", new VoiceMessageHander());
    	handlers.put("video", new VideoMessageHandler());
    	handlers.put("shortvideo", new ShortVideoMessageHandler());
    	handlers.put("location", new LocationHandler());
    }
	public boolean receiveMessage(String message) {
        boolean result=Boolean.TRUE;
		try {
			WechatMessageDTO wm = JSONObject.parseObject(message,WechatMessageDTO.class);
			String type = getMsgType(wm.content);
			MessageHandler handler=handlers.get(type);
			result = null!=handler && handler.onMessage(type,wm.entCode,wm.appId,wm.content);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return result;
    }

    private static synchronized String getMsgType(String message) throws DocumentException{
        String[] msgType=new String[2];
        Document doc = DocumentHelper.parseText(message);
        Node typeNode= doc.getRootElement().selectSingleNode("/xml/MsgType");
        Node eventNode= doc.getRootElement().selectSingleNode("/xml/Event");
        msgType[0]= typeNode!=null?typeNode.getText():"";
        msgType[1]= eventNode!=null?eventNode.getText():"";
        return join(msgType, "|");
    }

    private static String join(String[] strings, String separator) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < strings.length; i++) {
            String s=strings[i];
            if (i > 0 && null != s && !"".equals(s)) {
                sb.append(separator);
            }
            sb.append(strings[i]);
        }
        return sb.toString();
    }
}
